
__author__ = "Syun'ichi Shiraiwa"
__copyright__ = "Copyright, S. Shiraiwa, PiScope Project"
__credits__ = ["Syun'ichi Shiraiwa"]
__version__ = "0.5"
__maintainer__ = "S. Shiraiwa"
__email__ = "shiraiwa@psfc.mit.edu"
__status__ = "beta"

from ifigure.utils.debug import dprint

from ifigure.mto.fig_obj import FigObj
from ifigure.mto.fig_page import FigPage
from ifigure.widgets.canvas.file_structure import *
import ifigure
import os
import ifigure.utils.cbook as cbook
import ifigure.utils.geom as geom
import ifigure.widgets.canvas.custom_picker as cpicker
import numpy as np
import weakref
import logging
import matplotlib
import matplotlib.path
from matplotlib.patches import PathPatch

from ifigure.widgets.undo_redo_history import GlobalHistory
from ifigure.widgets.undo_redo_history import UndoRedoArtistProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjMethod

from ifigure.mto.generic_points import GenericPoint, GenericPointsHolder
from ifigure.mto.figobj_gpholder import FigObjGPHolder
from ifigure.utils.args_parser import ArgsParser
import matplotlib.transforms as mpltransforms

num_gp = 2


class FigArrow(FigObjGPHolder):
    def __new__(cls, *args, **kywds):
        def set_hidden_vars(obj):
            obj._objs = []  # for debug....
            obj._drag_backup = None
            obj._drag_mode = 0  # 1 transpose 2 expand 3 rotate
            obj._drag_start = None
            return obj

        if 'src' in kywds:
            obj = FigObjGPHolder.__new__(cls, *args, **kywds)
            obj = set_hidden_vars(obj)
            obj.setvar('clip', True)  # backword compatiblity should come here
            return obj

        p = ArgsParser()
        p.add_opt('x1', 0.0, ['number', 'dynamic'])
        p.add_opt('y1', 0.0, ['number', 'dynamic'])
        p.add_opt('x2', 1.0, ['number', 'dynamic'])
        p.add_opt('y2', 1.0, ['number', 'dynamic'])
        # special note
        # positions are first moved to _attr and then transferred
        # to gp point.
        # _attr['x1','y1'...] will not be saved.
        # Is it the best way? Also, are others such as done in
        # the same way?
        # Positions are managed by gp. So perhaps, this is not the
        # correct way....

        # attempting to clean this complexty (2014. 07/31)
        # [x1, x2,,] is not transferred to _attr
        # _eval_xy always return the value using getvar
        ###
        p.add_key('width', 1.0)
        p.add_key("arrowstyle", '-|>,head_length=10,head_width=5')
        p.add_key("clip", True)
        p.add_key("trans", ["figure", "figure"]*num_gp)
        p.add_key("transaxes", ["default"]*num_gp)
        p.add_key('draggable', True)

        v, kywds, d, flag = p.process(*args, **kywds)
        if not flag:
            raise ValueError('Failed when processing argument')

        if isinstance(v['trans'], str):
            v['trans'] = [v['trans'], ]*num_gp*2

        obj = FigObjGPHolder.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)

        for name in v:
            obj.setvar(name, v[name])

        if not 'facecolor' in kywds:
            kywds['facecolor'] = 'k'
        if not 'edgecolor' in kywds:
            kywds['edgecolor'] = 'k'
        obj.setvar("kywds", kywds)
        return obj

    def __init__(self, *args,  **kywds):
        '''
        Arrow object:
        '''
        self._cb_added = False

        args = []
        if 'src' not in kywds:
            kywds = self.getvar("kywds")

        GenericPointsHolder.__init__(self, num=2)
        super(FigArrow, self).__init__(*args, **kywds)

    @classmethod
    def isFigArrow(self):
        return True

    @classmethod
    def can_have_child(self, child=None):
        return False

    @classmethod
    def get_namebase(self):
        return 'arrow'

    @classmethod
    def property_in_file(self):
        return ["facecolor", "edgecolor", "alpha", "zorder"]

    @classmethod  # define _attr values to be saved
    def attr_in_file(self):
        return (["arrowstyle", "clip"] +
                super(FigArrow, self).attr_in_file())

    @classmethod
    def property_in_palette(self):
        return ["arrow", "patch"], [["arrowstyle"],
                                    ["facecolor", "edgecolor", "alpha"]]

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'arrow.png')
        return [idx1]

    def get_containter(self):
        return self.get_figpage()._artists[0]

#    def set_parent(self, parent):
#        FigObj.set_parent(self, parent)
#        GenericPointsHolder.set_gp_figpage(self)

    def canvas_menu(self):
        def onClipTextByAxes(evt, figobj=self):
            action1 = UndoRedoFigobjProperty(figobj._artists[0],
                                             'clip', not self.getp('clip'))
            window = evt.GetEventObject().GetTopLevelParent()
            hist = GlobalHistory().get_history(window)
            hist.make_entry([action1], menu_name='clipping')

        dprint('arrow_dragmode', self._drag_mode)
        if self._drag_mode == 1:
            m = self.gp_canvas_menu(self.get_gp(0))
        elif self._drag_mode == 2:
            m = self.gp_canvas_menu(self.get_gp(1))
        else:
            m = self.gp_holder_canvas_menu()

        if self.get_figaxes() is not None:
            if self.getp('clip'):
                txt = "^Clip by axes"
            else:
                txt = "*Clip by axes"
            m.extend([(txt, onClipTextByAxes, None), ])
        return m

    def isDraggable(self):
        return self._var["draggable"]

    def args2var(self):
        names0 = self.attr_in_file()
#        names  = ["x1","y1", "x2", "y2"]
#        use_np = [False]*4
        names = []
        use_np = []
        names = names + names0
        use_np = use_np + [False]*len(names0)
        values = self.put_args2var(names, use_np)

        self.set_gp_by_vars()

        x = values[0]
        y = values[1]
        if x is None:
            return False
        if y is None:
            return False
#        if (x.size != 1) or (y.size != 1):
#            self.setp("x", None)
#            self.setp("y", None)
#            return False
        return True

    def generate_artist(self):
        if self.isempty() is False:
            return

        lp = self.getp("loaded_property")

        # this will take care of use_var
        x1, y1, x2, y2 = self._eval_xy()
#        self._eval_xy()

        ###
        if lp is None:
            if x1 is not None:
                self.set_gp_point(0, x1, y1)
                self.set_gp_point(1, x2, y2)

#        if self.hasp("loaded_property"):
#            lp = self.getp("loaded_property")
#            while self.num_gp() < 2:
#            self.add_gp(GenericPoint(0,0,trans='figure'))
#            self.set_gp_figpage()
#            self.get_gp(0).dict_to_gp(lp[-1][0], self)
#            self.get_gp(1).dict_to_gp(lp[-1][1], self)

        a = self.make_newartist()
#        if self.hasp("loaded_property"):
#           self.set_artist_property(a, lp[0])
#           self.delp("loaded_property")
#
        if not self._cb_added:
            fig_page = self.get_figpage()
            fig_page.add_resize_cb(self)
            self._cb_added = True

    def onResize(self, evt):
        self.refresh_artist()

    def do_update_artist(self):
        self.refresh_artist()

    def make_newartist(self):
        self.check_loaded_gp_data()
        width = self.getp("width")
        arrowstyle = self.getp("arrowstyle")
        x1, y1 = self.get_gp(0).get_device_point()
        x2, y2 = self.get_gp(1).get_device_point()

        from matplotlib.transforms import IdentityTransform
        trans = IdentityTransform()
        styles = matplotlib.patches.ArrowStyle.get_styles()
        kywds = self.getvar('kywds')
        a = matplotlib.patches.FancyArrowPatch(posA=(x1, y1),
                                               posB=(x2, y2),
                                               connectionstyle='arc3',
                                               arrowstyle=arrowstyle, mutation_scale=1,
                                               transform=trans,
                                               **kywds)
        lp = self.getp("loaded_property")
        if lp is not None:
            self.set_artist_property(a, lp[0])
            self.delp("loaded_property")
        a.figobj = self
        a.figobj_hl = []
        a.set_zorder(self.getp('zorder'))
        self.add_artists(a)
        container = self.get_container()

        # this section works only when after artists are added
        if self.get_figaxes() is not None and self.getp('clip'):
            bbox = mpltransforms.Bbox.from_extents(
                container.get_window_extent().extents)
            try:
                self._artists[0].set_clip_box(bbox)
                # print self._artists[0].get_clip_box()
                self._artists[0].set_clip_on(True)
#                self._artists[0]._bbox_patch.set_clip_box(bbox)
#                self._artists[0]._bbox_patch.set_clip_on(True)
            except:
                pass
        self.delp("loaded_property")
        return a

    def add_artists(self, a):
        container = self.get_container()
        figure = self.get_figpage()._artists[0]
        self._artists = [a]
        if self.get_figaxes() is None:
            container.patches.append(a)
            a.set_figure(figure)
        else:
            container.add_patch(a)
            #a.set_figure(figure)
            #axes = self.get_figaxes()._artists[0]
            #a.set_axes(axes)

    def refresh_artist(self):
        a1 = self._artists[0]
        z = a1.zorder
        hl = len(self._artists[0].figobj_hl) != 0
        self.del_artist(delall=True)
        a = self.make_newartist()
        a.set_zorder(z)
#        self.add_artists(a)
        self.highlight_artist(hl, [a])
        a2 = self._artists[0]
        if a1 != a2:
            ifigure.events.SendReplaceEvent(self, a1=a1, a2=a2)

    def del_artist(self, artist=None, delall=False):
        if delall:
            artistlist = self._artists
        else:
            artistlist = artist

        val = []
        for a in self._artists:
            val.append(self.get_artist_property(a))
        if len(val) != 0:
            self.setp("loaded_property", val)

        if len(artistlist) != 0:
            #            container=self.get_figpage()._artists[0]
            container = self.get_container()
            is_figtext = self.get_figaxes() is None
            self.highlight_artist(False, artistlist)
            for a in artistlist:
                #            a is figure in this case
                if is_figtext:
                    container.patches.remove(a)
                else:
                    a.remove()

        super(FigArrow, self).del_artist(artistlist)

    def highlight_artist(self, val, artist=None):
        if artist is None:
            alist = self._artists
        else:
            alist = artist
        figure = self.get_figpage()._artists[0]
        w = figure.get_figwidth()*figure.get_dpi()
        h = figure.get_figheight()*figure.get_dpi()
        if val == True:
            for a in alist:
                box = a.get_window_extent().get_points()
                x = [box[0][0], box[0][0], box[1][0], box[1][0], box[0][0]]
                y = [box[0][1], box[1][1], box[1][1], box[0][1], box[0][1]]
                hl = matplotlib.lines.Line2D(x, y, marker='s',
                                             color='k', linestyle='None',
                                             markerfacecolor='none',
                                             markeredgewidth=0.5,
                                             figure=figure)
                figure.lines.extend([hl])
                a.figobj_hl.append(hl)
                if self._drag_mode == 1:
                    x, y = self.get_gp(0).get_device_point()
                    x = [x]
                    y = [y]
                    hl = matplotlib.lines.Line2D(x, y, marker='s',
                                                 color='k', linestyle='None',
                                                 markerfacecolor='k',
                                                 markeredgewidth=0.5,
                                                 figure=figure)
                    figure.lines.extend([hl])
                    a.figobj_hl.append(hl)
                if self._drag_mode == 2:
                    x, y = self.get_gp(1).get_device_point()
                    x = [x]
                    y = [y]
                    hl = matplotlib.lines.Line2D(x, y, marker='s',
                                                 color='k', linestyle='None',
                                                 markerfacecolor='k',
                                                 markeredgewidth=0.5,
                                                 figure=figure)
                    figure.lines.extend([hl])
                    a.figobj_hl.append(hl)
#              print 'if drag_mode is xxx highlight gp here', self._drag_mode
        else:
            for a in alist:
                for hl in a.figobj_hl:
                    figure.lines.remove(hl)
                    a.figobj_hl = []

    def get_artist_extent(self, a):
        box = a.get_window_extent().get_points()
        return [box[0][0], box[1][0], box[0][1], box[1][1]]

    def picker_a0(self, a, evt):
        from ifigure.widgets.canvas.custom_picker import linehit_test, abs_d
        hit, extra, type, loc = super(FigArrow, self).picker_a0(a, evt)
        type = 'area'
        loc = 3
#        hit2, extra =  a.contains(evt)
        hit2 = self.gp_hittest_line(evt, self.get_gp(0), self.get_gp(1))
        # print a, hit2, extra
        test = self.gp_hittest_rect(evt, self.get_gp(0), self.get_gp(1))
        # print hit2, test
        hit = False
        if test == 1:
            self._drag_mode = 1
            hit = True
            return hit, extra, type, loc
        elif test == 3:
            self._drag_mode = 2
            hit = True
            return hit, extra, type, loc
        elif test == 5 or hit2 == 1:
            hit = True
            self._drag_mode = 3
        if self.gp_hittest_p(evt, self.get_gp(0)):
            self._drag_mode = 1
            hit = True
        if self.gp_hittest_p(evt, self.get_gp(1)):
            self._drag_mode = 2
            hit = True
        return hit, {}, type, loc

    def drag_a(self, a, evt, shift=None, scale=None):
        redraw, scale0 = super(FigArrow, self).drag_a(a, evt,
                                                      shift=shift, scale=scale)
        if self._drag_mode == 1:
            x, y = self.get_device_point(1)
            self._drag_hl.set_xdata([evt.x, x])
            self._drag_hl.set_ydata([evt.y, y])
        if self._drag_mode == 2:
            x, y = self.get_device_point(0)
            self._drag_hl.set_xdata([evt.x, x])
            self._drag_hl.set_ydata([evt.y, y])
        return redraw, scale0

    def dragdone_a(self, a, evt, shift=None, scale=None):

        shift = evt.guiEvent_memory.ShiftDown()
        redraw, scale0 = super(FigArrow, self).dragdone_a(a, evt,
                                                          shift=shift, scale=scale)

        dx1 = 0
        dx2 = 0
        dy1 = 0
        dy2 = 0

        h = []
        if scale is None:
            dx = evt.x - self._st_p[0]
            dy = evt.y - self._st_p[1]
            if self._drag_mode & 1 != 0:
                dx1 = dx
                dy1 = dy
            if self._drag_mode & 2 != 0:
                dx2 = dx
                dy2 = dy
            a1 = self.move_gp_points(0, dx1, dy1, action=True)
            a2 = self.move_gp_points(1, dx2, dy2, action=True)
            h.append(a1)
            h.append(a2)
        else:
            h = h + self.scale_artist(scale, action=True)

        window = evt.guiEvent_memory.GetEventObject().GetTopLevelParent()
        hist = GlobalHistory().get_history(window)

        hist.start_record()
        for item in h:
            hist.add_history(item)
        hist.stop_record()
        return 0, scale0

    def scale_artist(self, scale, action=True):
        st_extent = self.get_artist_extent(self._artists[0])
        rec = geom.scale_rect(st_extent,
                              scale)
        gp = self.get_gp(0)
        x1, y1 = gp.get_device_point()
        gp = self.get_gp(1)
        x2, y2 = gp.get_device_point()

        if x1 > x2 and y1 > y2:
            i = (1, 3, 0, 2)
        elif x1 <= x2 and y1 > y2:
            i = (0, 3, 1, 2)
        elif x1 > x2 and y1 <= y2:
            i = (1, 2, 0, 3)
        elif x1 <= x2 and y1 <= y2:
            i = (0, 2, 1, 3)
        dx1 = rec[i[0]] - st_extent[i[0]]
        dy1 = rec[i[1]] - st_extent[i[1]]
        dx2 = rec[i[2]] - st_extent[i[2]]
        dy2 = rec[i[3]] - st_extent[i[3]]

        if action:
            h = []
            a1 = self.move_gp_points(0, dx1, dy1, action=True)
            a2 = self.move_gp_points(1, dx2, dy2, action=True)
            h.append(a1)
            h.append(a2)
            return h
        else:
            self.move_gp_points(0, dx1, dy1, action=False)
            self.move_gp_points(1, dx2, dy2, action=False)

    def set_arrowstyle(self, value, a):
        #        print 'set_arrowystyle', a, value
        self.set_update_artist_request()
        self.setp("arrowstyle", value)

    def get_arrowstyle(self, a):
        return self.getp("arrowstyle")

    def _eval_xy(self):
        if self.getp('use_var'):
            success = self.handle_use_var()
            if not success:
                return None, None, None, None
        return self.getvar('x1', 'y1', 'x2', 'y2')
#        if self.getp(("x1", "y1", "x2", "y2")) is None:
#            return None, None, None, None
#        return  self.getp(("x1", "y1", "x2", "y2"))



from ifigure.utils.cbook import parseStr
from ifigure.mto.py_code import PyData
from ifigure.mto.py_code import PyCode
import wx
import os
import re
import string
import logging
import shutil
from numpy import *
from collections import OrderedDict
from ifigure.mto.py_contents import Efitafile

######################################################
#         Setting for module file for py_module
#   name of module/class
module_name = 'EFIT_Pfile'
class_name = 'EFIT_Pfile'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#

menu = [("Import File...", "onLoadFile", True),
        ("Update Tree...", "onUpdateTree", True),
        ("+TextEditor...", None,  False),
        ("View Original File", "onOpenOrg", False),
        ("Edit File", "onOpenCurrent", False),
        ("!", None, False)]
method = ['init',
          'txt2tree',
          'onLoadFile',
          'onOpenOrg',  'onUpdateTree']

icon = 'data.png'
can_have_child = False
has_private_owndir = False
######################################################
modename = 'pfile_pathmode'
pathname = 'pfile_path'
extname = 'pfile_ext'


def init(self, *args, **kargs):
    if "src" not in kargs:
        self.onLoadFile(file='')

def process_lines(lines):
    import numpy as np
    l = [x.strip() for x in lines]
    counter = 0
    ret = {}

    while counter < len(l):
        ll = l[counter]; counter = counter + 1
        names = ll.split(' ')
        num = int(names[0])
        x = names[1]
        y = names[2]
        dy = names[3]
        val = []
        for i in range(num):
            val.append([float(x) for x in l[counter].split()])
            counter = counter + 1
        print(ll)
        ret[names[1]+' ' + names[2] + ' ' + names[3]] = np.vstack(val)
    return ret


def load_file(file=None):
    fid = open(file)
    lines = fid.readlines()
    fid.close()
    values = process_lines(lines)

    nm = Efitafile()
    nm["contents"] = values

    return nm


def load_afile(obj):
    file = obj.path2fullpath(modename=modename,
                             pathname=pathname)

    if file == '':
        return
    for name, child in obj.get_children():
        child.destroy()

    nm = load_file(file)
    obj.setvar0(nm)

def txt2tree(self, filename, txt):
    '''
    save txt to file and load it
    '''
    self_obj = self.td
    if not self_obj.has_owndir():
        self_obj.mk_owndir()

    if os.path.basename(filename) == filename:
        path = os.path.join(self_obj.owndir(), filename)
    else:
        path = filename

    fid = open(path, 'w')
    fid.write(txt)
    fid.close()

    self.td.set_path_pathmode(path, modename, pathname, extname)
    load_afile(self.td)


def onUpdateTree(self, event=None):
    load_pfile(self.td)


def onLoadFile(self, e=None, file=''):
    from ifigure.utils.addon_utils import onLoadFile

    if file != '':
        self.td.set_path_pathmode(file, modename, pathname, extname)
        ret = True
    else:
        ret = onLoadFile(self.td, message="Select P-File",
                         modename=modename,
                         pathname=pathname,
                         extname=extname)
    if not ret:
        return False

    load_afile(self.td)


def onOpenOrg(self, e=None):
    from ifigure.utils.addon_utils import onOpenOrg
    onOpenOrg(self.td)


def onOpenCurrent(self, e=None):
    from ifigure.utils.addon_utils import onOpenCurrent
    onOpenCurrent(self.td, modename=modename,
                  pathname=pathname)



from __future__ import print_function
import wx
import sys
import weakref
import wx.aui as aui
from ifigure.utils.cbook import ImageFiles, Write2Main, BuildPopUpMenu
from ifigure.widgets.var_viewerg import VarViewerG, VarViewerGDropTarget
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
from ifigure.utils.edit_list import EditListPanel
from ifigure.mto.fig_obj import FigObj
import ifigure.events


class AutoWidthListCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(
            self, parent, -1, style=wx.LC_REPORT | wx.LC_EDIT_LABELS)
        ListCtrlAutoWidthMixin.__init__(self)


class ProjViewerDropTarget(wx.TextDropTarget):
    # obj is proj_viewer
    def __init__(self, obj):
        wx.TextDropTarget.__init__(self)
        self.obj = obj
        self._prev_item = None
        self._prev_dict = None
        self._timer_on = False
        self._motion_on = False
        self._item_at_start = None
        self._item_at_starte = None

    def OnTime(self, e):
        #        print "timer"
        x, y = self.obj.tree.ScreenToClient(wx.GetMousePosition())
        size = self.obj.tree.GetSize()
        if y < 0:
            self.ScrollUp()
            if y < -10:
                self.StopTimer()
            return
        elif y > size[1]:
            self.ScrollDown()
            if y > size[1]+10:
                self.StopTimer()
            return

        self._timer_on = False
        if self._item_at_start is not None:
            # collapse this item
            # print "collapse"
            self.obj.tree.Collapse(self._item_at_start)
        if self._item_at_starte is not None:
            # collapse this item
            # print "expand"
            self.obj.tree.Expand(self._item_at_starte)
        self._item_at_start = None
        self._item_at_starte = None

    def OnDragOver(self, x, y, default):
        item, flag = self.obj.tree.HitTest((x, y))

#        print flag & wx.TREE_HITTEST_ONITEMLABEL
        if ((flag & wx.TREE_HITTEST_ONITEMINDENT) or
            (flag & wx.TREE_HITTEST_ONITEMBUTTON) or
                (flag & wx.TREE_HITTEST_TOLEFT)):
            if self._timer_on == False:
                self.obj.tree.Bind(wx.EVT_TIMER, self.OnTime)
                if not hasattr(self, 'timer'):
                    self.timer = wx.Timer(self.obj.tree, wx.ID_ANY)
                    self.timer.Start(100)
                self._timer_on = True
                self._item_at_start = item
        else:
            # cancel action
            self._item_at_start = None

        if flag & wx.TREE_HITTEST_ONITEMLABEL:
            dictobj = self.obj.tree.GetPyData(item)
            if self._prev_item is not None:
                self.obj.tree.SetItemDropHighlight(self._prev_item, False)
            self.obj.tree.SetItemDropHighlight(item)
            if self._prev_dict != dictobj:
                self.obj.varviewer.fill_list(dictobj)
            self._prev_item = item
            self._prev_dict = dictobj
            if self._timer_on == False:
                self.obj.tree.Bind(wx.EVT_TIMER, self.OnTime)
                if not hasattr(self, 'timer'):
                    self.timer = wx.Timer(self.obj.tree, wx.ID_ANY)
                self.timer.Start(100)
                self._timer_on = True
                self._item_at_starte = item

        else:
            pass

        return default

    def OnLeave(self):
        if self._prev_item is not None:
            self.obj.tree.SetItemDropHighlight(self._prev_item, False)

    def OnEnter(self, x, y, default):
        if self._motion_on == False:
            #          self.obj.tree.Bind(wx.EVT_MOTION, self.OnMotion)
            pass

        self._motion_on = True
        return default

    def StopTimer(self):
        print("stop timer")
        if self.timer is not None:
            self.timer.Stop()
            del self.timer
        self._prev_item = None
        self._prev_dict = None
        self._timer_on = False
        self._motion_on = False
        self._item_at_start = None
        self._item_at_starte = None

    def OnDropText(self, x, y, data):
        self.StopTimer()
        try:
            app = self.obj.tree.GetTopLevelParent()
            data = app._text_clip
            app._text_clip = ''

        except Exception:
            print("error in processing dnd in wx.tree")
            pass
        if self._prev_item is not None:
            self.obj.tree.SetItemDropHighlight(self._prev_item, False)
        self._prev_item = None
        data = str(data)

    def ScrollUp(self):
        if "wxMSW" in wx.PlatformInfo:
            self.obj.tree.ScrollLines(-1)
        else:
            first = self.obj.tree.GetFirstVisibleItem()
            prev = self.obj.tree.GetPrevSibling(first)
            if prev:
                # drill down to find last expanded child
                while self.obj.tree.IsExpanded(prev):
                    prev = self.obj.tree.GetLastChild(prev)
            else:
                # if no previous sub then try the parent
                prev = self.obj.tree.GetItemParent(first)

            if prev:
                self.obj.tree.ScrollTo(prev)
            else:
                self.obj.tree.EnsureVisible(first)

    def ScrollDown(self):
        if "wxMSW" in wx.PlatformInfo:
            self.obj.tree.ScrollLines(1)
        else:
            # first find last visible item by starting with the first
            next = None
            last = None
            item = self.obj.tree.GetFirstVisibleItem()
            while item:
                if not self.obj.tree.IsVisible(item):
                    break
                last = item
                item = self.obj.tree.GetNextVisible(item)

            # figure out what the next visible item should be,
            # either the first child, the next sibling, or the
            # parent's sibling
            if last:
                if self.obj.tree.IsExpanded(last):
                    next = self.obj.tree.GetFirstChild(last)[0]
                else:
                    next = self.obj.tree.GetNextSibling(last)
                    if not next:
                        prnt = self.obj.tree.GetItemParent(last)
                        if prnt:
                            next = self.obj.tree.GetNextSibling(prnt)

            if next:
                self.obj.tree.ScrollTo(next)
            elif last:
                self.obj.tree.EnsureVisible(last)


class ProjTreeViewerPopUp(wx.Menu):
    def __init__(self, parent, dictobj):
        super(ProjTreeViewerPopUp, self).__init__()
        self.parent = parent
        menus = dictobj.tree_viewer_menu()
        menus = menus+[('---', None, None),
                       ('ExpandAll', self.onExpandChild, None),
                       ('Copy Path', self.onCopyPath, None),
                       ('Refresh', self.onRefresh, None)]

        BuildPopUpMenu(self, menus)
        return

    def onRefresh(self, e):
        self.parent.update_widget()

    def onExpandChild(self, e):
        tv = (e.GetEventObject()).parent
        item = tv.tree.GetSelection()
        tv.tree.ExpandAllChildren(item)

    def onCopyPath(self, e):
        tv = (e.GetEventObject()).parent
        item = tv.tree.GetSelection()
        dictobj = tv.tree.GetPyData(item)
        tx = 'ifigure.'+dictobj.get_full_path()
        app = tv.GetTopLevelParent()
        app.shell.CopyText(len(tx), tx)


class ProjTreeViewer(wx.Panel):
    def __init__(self, parent=None):
        """Constructor"""

        super(ProjTreeViewer, self).__init__(parent)
        self._drag_start = False
        self.panel = None

#        sizer=wx.BoxSizer(wx.VERTICAL)
#        self.SetSizer(sizer)
#        self.splitter=wx.SplitterWindow(self)
#        sizer.Add(self.splitter, 1, wx.EXPAND)
#        self.nb = aui.AuiNotebook(self, style=aui.AUI_NB_TAB_SPLIT|
#                                              aui.AUI_NB_TAB_MOVE|
#                                              aui.AUI_NB_SCROLL_BUTTONS)

        ### make tree ###
        self.tree = wx.TreeCtrl(self, wx.ID_ANY,
                                wx.DefaultPosition,
                                (-1, -1), wx.TR_HAS_BUTTONS)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.tree, 1, wx.EXPAND)
        self.SetSizer(sizer)
#        self.nb.AddPage(self.tree, 'Project Tree')
        self.tree.parent = self
        self.dt1 = ProjViewerDropTarget(self)
        self.tree.SetDropTarget(self.dt1)

        im = ImageFiles()
        self.tree.SetIndent(8)
        self.tree.SetImageList(im.get_imagelist())

        self.Bind(wx.EVT_TREE_ITEM_EXPANDING, self.OnExpandItem)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSING, self.OnCollapseItem)
        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnItemRightClick)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)
        self.Bind(wx.EVT_TREE_SEL_CHANGING, self.OnSelChanging)
        self.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnBeginDrag)
        self.Bind(wx.EVT_TREE_END_DRAG, self.OnEndDrag)
        self.Bind(wx.EVT_MENU, self.OnMenu)
        self.Bind(wx.EVT_TREE_KEY_DOWN, self.OnKeyDown)
        self.tree.GetTopLevelParent().Bind(wx.EVT_MOUSE_CAPTURE_LOST, lambda x: None)

    def init_sash_pos(self):
        return
        # self.splitter.SetSashPosition(200)

    def get_proj(self):
        app = self.GetTopLevelParent()
        return app.proj

    def update_widget(self):
        sel = self.tree.GetSelection()
        if sel is not None:
            sdict = self.tree.GetPyData(sel)
        else:
            sdict = None

        proj = self.get_proj()
        if proj is None:
            self.tree.DeleteAllItems()
            return
        if sdict is None:
            sdict = proj  # default select proj

        croot = self.tree.GetRootItem()
        if self.tree.GetPyData(croot) is None:
            self.tree.DeleteAllItems()
            croot = self.tree.AddRoot(proj.name)
            self.tree.SetPyData(croot,  self.get_proj())

        #
        #  remove item from tree viewer
        #
        while self.remove_item_from_tree():
            pass
        #
        #  add item to tree viewer
        #
        while self.add_item_to_tree():
            pass
        #
        #  refresh tree name...
        #
        for item in self.walk_treectrl(croot):
            treedict = self.tree.GetPyData(item)
            name = treedict.name
            if treedict.status != '':
                label = name+' ('+treedict.status+')'
            else:
                label = name
            self.tree.SetItemText(item, label)

        self.varviewer.fill_list(sdict)
        return

    def walk_treectrl(self, item):
        yield item
        (child, cookie) = self.tree.GetFirstChild(item)
        while child.IsOk():
            for x in self.walk_treectrl(child):
                yield x
            (child, cookie) = self.tree.GetNextChild(item, cookie)

    def OnButton(self, e):
        pass

    def OnKeyDown(self, e):
        "map delete to selcting delete in pull down"
        if e.GetKeyCode() == 8:
            item = self.tree.GetSelection()
            dictobj = self.tree.GetPyData(item)
            dictobj.onDelete(e)

            if isinstance(dictobj, FigObj):
                self.GetTopLevelParent().draw()
            else:
                self.update_widget()

    def OnExpandItem(self, e):
        #        print "change visible (expand)"
        item = e.GetItem()
        dictobj = self.tree.GetPyData(item)
        if dictobj is None:
            return
        for name, child in dictobj.get_children():
            child.set_visible(True)

    def OnCollapseItem(self, e):
        #        print "change visible (collapse)"
        item = e.GetItem()
        dictobj = self.tree.GetPyData(item)
        if dictobj is None:
            return
        for name, child in dictobj.get_children():
            child.set_visible(False)

    def OnItemRightClick(self, e):
        item = e.GetItem()
        dictobj = self.tree.GetPyData(item)
        if dictobj is None:
            return
        menus = dictobj.tree_viewer_menu()
        if menus is None:
            return None

        m = ProjTreeViewerPopUp(self, dictobj)
        self.PopupMenu(m,
                       e.GetPoint())
        m.Destroy()

    def OnSelChanged(self, e):
        item = e.GetItem()
        dictobj = self.tree.GetPyData(item)
        if dictobj is None:
            return
        self.varviewer.fill_list(dictobj)
        if isinstance(dictobj, FigObj):
            if len(dictobj._artists) != 0:
                sel = [weakref.ref(dictobj._artists[0])]
                ifigure.events.SendSelectionEvent(dictobj, self, sel)

    def OnSelChanging(self, e):
        if self._drag_start:
            self._drag_start = False
            e.Veto()

    def OnMenu(self, e):
        self.update_widget()

    def OnBeginDrag(self, e):
        item = e.GetItem()
        dictobj = self.tree.GetPyData(item)
#        self.dragItem = e.GetItem()
        app = self.tree.GetTopLevelParent()
        app._text_clip = dictobj.get_full_path()
        text = dictobj.get_full_path() if six.PY3 else unicode(dictobj.get_full_path())
        tdo = wx.TextDataObject(text)
        tds = wx.DropSource(self.tree)
        tds.SetData(tdo)
        tds.DoDragDrop(True)
#        self.tree.Unselect()
        self._drag_start = True
        e.Skip()

    def OnEndDrag(self, e):
        print("OnEndDrag")
#        self.tree.Unselect()
        Write2Main(e, "ev")
        if not e.GetItem().IsOk():
            return
        # Make sure this memeber exists.
        try:
            old = self.dragItem
        except:
            return

        # Get the other IDs that are involved
        new = e.GetItem()
        parent = self.GetItemParent(new)
        if not parent.IsOk():
            return
        text = self.GetItemText(old)
#        self.Delete(old)
#        self.InsertItem(parent, new, text)

    def OpenPanel(self, list, obj=None, callback=None):
        if self.panel is not None:
            self.ClosePanel()
        self.panel = wx.Panel(self, wx.ID_ANY)
        vbox = wx.BoxSizer(wx.VERTICAL)
        self.panel.SetSizer(vbox)
        self.elp = EditListPanel(self.panel, list)
        vbox.Add(self.elp, 0, wx.EXPAND |
                 wx.ALIGN_CENTER | wx.RIGHT | wx.LEFT, 1)
        button = wx.Button(self.panel, wx.ID_ANY, "Apply")
        vbox.Add(button, 0, wx.ALIGN_RIGHT | wx.ALIGN_TOP)
        button.Bind(wx.EVT_BUTTON, self.onPanelApply)
#        vbox.SetSizeHints(self)
#        elp.GetSizer().SetSizeHints(self)

        sizer = self.GetSizer()
        sizer.Add(self.panel, 1, wx.EXPAND)
        self.GetTopLevelParent()._force_layout()
        self.callback = callback
        self.callback_obj = weakref.proxy(obj, self.onPanelClean)

    def onPanelApply(self, e):
        if self.callback_obj is not None:
            m = getattr(self.callback_obj, self.callback)
            m(self.elp.GetValue())
            self.ClosePanel()
        else:
            self.ClosePanel()

    def onPanelClean(self, ref):
        if self.panel is not None:
            self.ClosePanel()
            print('panel clean')

    def ClosePanel(self):
        self.GetSizer().Remove(self.panel)
        self.panel.Destroy()
        self.GetTopLevelParent()._force_layout()
        self.panel = None

    def onTD_Selection(self, evt):
        if len(evt.selections) < 2:
            croot = self.tree.GetRootItem()
            t0 = evt.GetTreeDict()
            print(t0)
            for item in self.walk_treectrl(croot):
                if t0 == self.tree.GetPyData(item):
                    break

            # unbind temprorary to avoid change of event generation
            self.Unbind(wx.EVT_TREE_SEL_CHANGED)
            self.tree.SelectItem(item)
            dictobj = self.tree.GetPyData(item)
            if dictobj is not None:
                self.varviewer.fill_list(dictobj)
            self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged)

    def remove_item_from_tree(self):
        croot = self.tree.GetRootItem()
        for item in self.walk_treectrl(croot):
            treedict = self.tree.GetPyData(item)
            print(treedict.get_full_path())
            print(self.item_path(item))
            if treedict is None:
                # rm_list.append(item)
                self.tree.Delete(item)
                return True
            if (treedict.get_full_path() !=
                    self.item_path(item)):
                self.tree.Delete(item)
                return True

            if (treedict.get_parent() is None and
                    treedict is not self.get_proj()):
                self.tree.Delete(item)
                return True

        return False

    def item_path(self, item):
        path = []
        p = item
        croot = self.tree.GetRootItem()
        while p != croot:
            path = [self.tree.GetItemText(p)]+path
            p = self.tree.GetItemParent(p)
        path = [self.tree.GetItemText(p)]+path
        return '.'.join(path)

    def add_item_to_tree(self):
        croot = self.tree.GetRootItem()
        oge = self.walk_treectrl(croot)
        nge = self.get_proj().walk_tree()

        ntd = self.get_proj()
        otd = ntd
        ntd2 = ntd
        otd2 = ntd
        oitem = croot
        while True:
            try:
                ntd2 = next(nge)
                try:
                    oitem2 = next(oge)
                    otd2 = self.tree.GetPyData(oitem2)
                except Exception:
                    otd2 = None

                if ntd2 is not otd2:
                    #              if otd2 is not None: print otd2.get_full_

                    if ntd2.get_parent() is ntd:
                        #                 print "adding first child", ntd2.get_full_path()
                        self.fill_sub_tree(oitem, ntd2,
                                           sel_dict=None, first=True)
                        break

                    pitem = self.tree.GetItemParent(oitem)
                    while (self.tree.GetPyData(pitem)
                           is not ntd2.get_parent()):
                        oitem = pitem
                        pitem = self.tree.GetItemParent(pitem)

#              print "adding sibling", ntd2.get_full_path()
#              print "next to", self.tree.GetPyData(oitem).get_full_path()
                    self.fill_sub_tree(
                        pitem,
                        ntd2,
                        sel_dict=None, sib=oitem)
                    break
                ntd = ntd2
                otd = otd2
                oitem = oitem2
            except StopIteration:
                return False

        return True

    def fill_sub_tree(self, pitem, treedict, sel_dict=None,
                      sib=None, first=False):

        name = treedict.name
        if treedict.status != '':
            label = name+' ('+treedict.status+')'
        else:
            label = name
        img = treedict.classimage()
        if sib is None:
            if first is True:
                parent2 = self.tree.InsertItemBefore(pitem, 0,
                                                     label, img)
            else:
                parent2 = self.tree.AppendItem(pitem, label, img)
        else:
            parent2 = self.tree.InsertItem(pitem, sib, label, img)
        if treedict is sel_dict:
            self.tree.SelectItem(parent2)
        if treedict.is_suppress():
            self.tree.SetItemTextColour(parent2,
                                        wx.NamedColour('Grey'))
        if treedict.can_have_child():
            self.tree.SetItemHasChildren(parent2)

        self.tree.SetPyData(parent2, treedict)
        if treedict.isTreeLink():
            l = treedict.get_linkobj()
            if l is not None:
                tx = l.name
            else:
                tx = 'None'
            self.tree.SetItemText(parent2,
                                  '-> '+name + '(' + tx + ')')
        else:
            for n2, child in treedict.get_children():
                self.fill_sub_tree(parent2, child,
                                   sel_dict=sel_dict)
                # self.tree.Expand(parent2)

        if treedict.is_visible():
            self.tree.Expand(pitem)



def AddSysPath(new_path):
    """ AddSysPath(new_path): adds a directory to Python's sys.path

    Does not add the directory if it does not exist or if it's already on
    sys.path. Returns 1 if OK, -1 if new_path does not exist, 0 if it was
    already on sys.path.
    """
    import sys
    import os

    # Avoid adding nonexistent paths
    if not os.path.exists(new_path):
        return -1

    # Standardize the path. Windows is case-insensitive, so lowercase
    # for definiteness.
    new_path = os.path.abspath(new_path)
    if sys.platform == 'win32':
        new_path = new_path.lower()

    # Check against all currently available paths
    for x in sys.path:
        x = os.path.abspath(x)
        if sys.platform == 'win32':
            x = x.lower()
        if new_path in (x, x + os.sep):
            return 0
    sys.path.append(new_path)
    return 1



#
#  these routines recovers old scipy interp1d and interp2d 
#
from scipy.interpolate import RectBivariateSpline, make_interp_spline

def interp1d(x, y, kind='linear'):
    if kind=='linear':
        return make_interp_spline(x, y, k=1)
    elif kind=='cubic':
        return make_interp_spline(x, y, k=3)    


def interp2d(x, y, z, kind='linear'):
    if kind == 'cubic':
        kx = 3
        ky = 3
    elif kind == 'linear':
        kx = 1
        ky = 1
        
    return RectBivariateSpline(x, y, z.transpose(), kx=kx, ky=ky)



from __future__ import print_function
#  Name   :py_content
#
#      base class for contents, in which
#      tree datas are imported
#
#      it includes...
#        1) Namelist
#        2) EFIT Gfile
#        3) netcdf4
#        4) MDS plus tree
#
#  Author :
#         Syun'ichi Shiraiwa
#  E-mail :
#         shiraiwa@psfc.mit.edu
#
#
#  History :
#         was born sometime around 2012. 04
#         2012. 08 still keeps growing...
#
# *******************************************
#     Copyright(c) 2012- S. Shiraiwa
# *******************************************

import collections
import weakref
import sys
import time
from six.moves import queue as Queue
import ifigure.events
from ifigure.widgets.var_viewerg2 import VarViewerGValue
from ifigure.utils.cbook import SetText2Clipboard


class PyContents(collections.OrderedDict):
    def __init__(self, *arg, **karg):
        #       self._var0=None
        super(PyContents, self).__init__(*arg, **karg)

    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = PyContents(value)
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)


class NamelistVVV(VarViewerGValue):
    def get_drag_text2(self, key):
        ''' 
        text for dnd from var viewer
        '''
        text = self._path+',"'+key+'")'
        return text

    def OnCompareItems(self, t1, t2):
        return (list(t1[0]._var0.keys()).index(t1[1][0]) -
                list(t2[0]._var0.keys()).index(t2[1][0]))


class Namelist(PyContents):
    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = self.__class__(value)
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)

    def __getitem__(self, key):
        '''
        namelist ignores case
        '''
        if key in self:
            return PyContents.__getitem__(self, key)
        elif key.upper() in self:
            return PyContents.__getitem__(self, key.upper())
        elif key.lower() in self:
            return PyContents.__getitem__(self, key.lower())
        else:
            raise KeyError('Key is not found :'+key)

    def show_contents(self, tree, td, item, base=None, keylist=None):
        ''' 
        proj viwer helper method
        '''
        img = td.get_classimage()

        if base is None:
            base = self
        if keylist is None:
            keylist = []
        pitem = item
        for key in base:
            tree.SetItemHasChildren(pitem)
            pitem2 = tree.AppendItem(pitem, key, img)
            vv_val = self.get_varviewer_value(td, key)
            tree.SetPyData(pitem2, (td, (key, base[key]), vv_val))

    def hide_contents(self, tree, td, item):
        tree.DeleteChildren(item)
        tree.SetItemHasChildren(item, False)

    def get_drag_text1(self, pydata):
        ''' 
        text for dnd from proj viewer
        '''
        return pydata[0].get_full_path()+'.get_contents("'+pydata[1][0]+'")'

    def get_varviewer_value(self, td, key):
        var = td._var0[key]
        note = {}
        val = NamelistVVV(var, note)
        if isinstance(key, list):
            txt = '","'.join(key)
        else:
            txt = key
        val._path = td.get_full_path()+'.get_contents("'+txt+'"'
        return val


class MatfileVVV(NamelistVVV):
    pass


class MatData(dict):
    def __repr__(self):
        dataname = ['  ' + x for x in self if not x.startswith('__')]
        return "Matlab (.mat) data (derived from dict class). \n" + '\n'.join(dataname)


class Matfile(Namelist):
    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = Matfile(value)
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)

    def get_varviewer_value(self, td, key):
        var = td._var0[key]
        note = {}
        val = MatfileVVV(var, note)
        if isinstance(key, list):
            txt = '","'.join(key)
        else:
            txt = key
        val._path = td.get_full_path()+'.get_contents("'+txt+'"'
        return val


class IDLfileVVV(NamelistVVV):
    pass


class IDLData(PyContents):
    def __repr__(self):
        dataname = ['  ' + x for x in self if not x.startswith('__')]
        return "IDL (.sav) data (derived from dict class). \n" + '\n'.join(dataname)


class IDLfile(Namelist):
    def __init__(self, *arg, **karg):
        self._var0 = {}
        super(IDLfile, self).__init__(*arg, **karg)

    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = IDLfile(value)
        else:
            value2 = value
        if hasattr(value, '_var0'):
            value2._var0 = value._var0
        super(PyContents, self).__setitem__(key, value2)

    def __getitem__(self, key):
        '''
        IDL data is either in _var0 or subtree(child)
        '''
        try:
            return PyContents.__getitem__(self, key)
        except:
            try:
                return self._var0.__getitem__(key)
            except:
                raise

    def show_contents(self, tree, td, item, base=None, keylist=None):
        ''' 
        proj viwer helper method
        '''
        img = td.get_classimage()

        if base is None:
            base = self
        if keylist is None:
            keylist = []
        self.make_contents_branch(tree, td, item, self, img)

    def make_contents_branch(self, tree, td, pitem, d, img,
                             keylist=None):
        if keylist is None:
            keylist = []
        for key in list(d.keys()):
            tree.SetItemHasChildren(pitem)
            pitem2 = tree.AppendItem(pitem, key, img)
            vv_val = self.get_varviewer_value(td, keylist+[key])
            tree.SetPyData(pitem2, (td, (keylist+[key], d[key]), vv_val))
            if isinstance(d[key], dict):
                keylist2 = [x for x in keylist]
                keylist2.append(key)
                self.make_contents_branch(tree, td, pitem2, d[key], img,
                                          keylist=keylist2)

    def get_drag_text1(self, pydata):
        ''' 
        text for dnd from proj viewer
        '''
        txt = '","'.join(pydata[1][0])
        return pydata[0].get_full_path()+'.get_contents("'+txt+'")'

    def get_varviewer_value(self, td, keylist):
        p = td[:]
        for k in keylist:
            p = p[k]

        var = {x: p[x] for x in list(p.keys()) if not isinstance(p[x], IDLfile)}

        note = {}
        val = IDLfileVVV(p._var0, note)
        val._td = td
        if isinstance(keylist, list):
            txt = '","'.join(keylist)
        else:
            txt = key
        val._path = td.get_full_path()+'.get_contents("'+txt+'"'
        return val


class EfitgfileVVV(NamelistVVV):
    pass


class Efitgfile(Namelist):
    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = Efitgfile(value)
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)

    def get_varviewer_value(self, td, key):
        var = td._var0[key]
        note = {}
        val = EfitgfileVVV(var, note)
        if isinstance(key, list):
            txt = '","'.join(key)
        else:
            txt = key
        val._path = td.get_full_path()+'.get_contents("'+txt+'"'
        return val


class EfitafileVVV(NamelistVVV):
    pass


class Efitafile(Namelist):
    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = Efitafile(value)
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)

    def get_varviewer_value(self, td, key):
        var = td._var0[key]
        note = {}
        val = EfitafileVVV(var, note)
        if isinstance(key, list):
            txt = '","'.join(key)
        else:
            txt = key
        val._path = td.get_full_path()+'.get_contents("'+txt+'"'
        return val


class MDSPlusTreeVVV(NamelistVVV):
    def get_drag_text2(self, key):
        ''' 
        text for dnd from var viewer
        '''
        text = self._td.get_full_path()+self._path+'.mds_eval("'+key + \
            '",'+self._td.get_full_path()+')'
        return text

    def OnCompareItems(self, t1, t2):
        klist = t1[1][0]
        p1 = t1[0]._var0
        for key in klist[:-1]:
            p1 = p1[key]
        p2 = t2[0]._var0
        klist = t2[1][0]
        for key in klist[:-1]:
            p2 = p2[key]

        return (list(p1.keys()).index(t1[1][0][-1]) -
                list(p2.keys()).index(t2[1][0][-1]))

    def tree_viewer_menu(self):
        if self.hasvar('mds_path'):
            return [("Show in MDSScope (1D)",  self.onScope1D, None),
                    ("Show in MDSScope (image)",  self.onScope2Di, None),
                    ("Show in MDSScope (contour)",  self.onScope2Dc, None),
                    ("Show in MDSScope (surface)",  self.onScope2Ds, None), ]

        else:
            return []

    def onScope1D(self, evt):
        from ifigure.mdsplus.fig_mds import FigMds

        path = self.getvar('mds_path').strip()
        obj = FigMds()
        obj.setvar("experiment", str(self._td.getvar('tree')))
        obj.setvar("default_node", '')
        data = collections.OrderedDict()
#       data["init"] = ''
        data["x"] = 'dim_of('+path+')'
        data["y"] = path
        data["z"] = ''
        data["xerr"] = ''
        data["yerr"] = ''
        obj.setvar("mdsvars", data)
        obj.setvar("title", '')
        obj.setvar("update", '')
        self._on_scope(obj)

    def onScope2Di(self, evt):
        self._on_scope_2D(evt, 'image', False)

    def onScope2Dc(self, evt):
        self._on_scope_2D(evt, 'contour', False)

    def onScope2Ds(self, evt):
        self._on_scope_2D(evt, 'surface', True)

    def _on_scope_2D(self, evt, type, threed):
        from ifigure.mdsplus.fig_mds import FigMds

        path = self.getvar('mds_path').strip()
        obj = FigMds()
        obj._plot_type = type
        obj.setvar("experiment", str(self._td.getvar('tree')))
        obj.setvar("default_node", '')
        data = collections.OrderedDict()
#       data["init"] = ''
        data["x"] = 'dim_of('+path+')'
        data["y"] = 'dim_of('+path+', 1)'
        data["z"] = path
        data["xerr"] = ''
        data["yerr"] = ''
        obj.setvar("mdsvars", data)
        obj.setvar("title", '')
        obj.setvar("update", '')
        self._on_scope(obj, threed)

    def _on_scope(self, obj, threed=False):
        #       print self.getvar('mds_path')
        #       print self._td.get_full_path(), self._path
        from ifigure.interactive import scope

#       book = scope()
        viewer = scope()
        book = viewer.book
        viewer.set_mdsshot(self._td.getvar('shot'))
        p = book.get_page(0)
        a = p.get_axes(0)
        a.set_3d(threed)
        name = a.get_next_name(obj.get_namebase())
        a.add_child(name, obj)
        obj.realize()

        ifigure.events.SendPVAddFigobj(a)
        ifigure.events.SendChangedEvent(a, w=viewer.canvas)


class MDSPlusTree(Namelist):
    #   def __init__(self, top, *arg, **karg):
    #       self._td = weakref.ref(top)
    def __init__(self, *arg, **karg):
        super(Namelist, self).__init__(*arg, **karg)

    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            #          value2=MDSPlusTree(self._td(), value)
            value2 = MDSPlusTree(value)
            if hasattr(value, 'mds_path'):
                value2.mds_path = value.mds_path
        else:
            value2 = value
        super(PyContents, self).__setitem__(key, value2)

    def show_contents(self, tree, td, item, base=None, keylist=None):
        ''' 
        proj viwer helper method
        '''
        img = td.get_classimage()

        if base is None:
            base = self
        if keylist is None:
            keylist = []
        self.make_contents_branch(tree, td, item, self, img)

    def make_contents_branch(self, tree, td, pitem, d, img,
                             keylist=None):
        if keylist is None:
            keylist = []
        for key in list(d.keys()):
            tree.SetItemHasChildren(pitem)
            pitem2 = tree.AppendItem(pitem, key, img)
            vv_val = self.get_varviewer_value(td, keylist+[key])
            tree.SetPyData(pitem2, (td, (keylist+[key], d[key]), vv_val))
            if isinstance(d[key], dict):
                keylist2 = [x for x in keylist]
                keylist2.append(key)
                self.make_contents_branch(tree, td, pitem2, d[key], img,
                                          keylist=keylist2)

    def get_varviewer_value(self, td, keylist):
        p = td[:]
        for k in keylist:
            p = p[k]
        if hasattr(p, 'mds_path'):
            val = MDSPlusTreeVVV({"value": "*", "mds_path": p.mds_path},  {})
        else:
            val = MDSPlusTreeVVV({"value": "*"},  {})
        txt = '","'.join(keylist)
        if not isinstance(td, weakref.ProxyType):
            td = weakref.proxy(td)
        val._td = td
        val._path = '.get_contents("'+txt+'")'
        return val

    def get_drag_text1(self, pydata):
        ''' 
        text for dnd from proj viewer
        '''
        txt = '","'.join(pydata[1][0])
        return pydata[0].get_full_path()+'.get_contents("'+txt+'")'

    def get_info(self):
        print(('getting info', self.mds_path))

    def eval(self, td, **kargs):
        return self.mds_eval(td, **kargs)

    def mds_eval(self, td, key='value', maxwait=60, shot=None, debug=False):
        '''
            key : 'value', 'dim0', 'dim1'
        '''
#       import ifigure.utils.mdsplusr as mds
        if debug:
            print(('eval', key, self.mds_path))

#       td = self._td()
        if key == "value":
            expr = self.mds_path
        elif key == "dim0":
            expr = 'dim_of('+self.mds_path+',0)'
        elif key == "dim1":
            expr = 'dim_of('+self.mds_path+',1)'

        tree = td.getvar("tree")
        if shot is None:
            shot = td.eval("shot")
        server = td.getvar('mdsplus_server')
        if server is None:
            server = td.get_parent().getvar('mdsplus_server')
        from ifigure.mdsplus.mdsscope import MDSWorkerPool

        if MDSWorkerPool().pool is None:
            MDSWorkerPool().Reset()
        pool = MDSWorkerPool().pool

#       w, wid = pool.get_worker()
#       if w is None:
#           print 'worker not available'
#           return

        from ifigure.mdsplus.fig_mds import MDSsession
        from ifigure.mdsplus.mds_job import MDSjob
        from ifigure.mdsplus.mdsscope import AnaGroup, GlobalCounter

        ana = MDSsession()
        job0 = MDSjob('connection_mode', server)
        ana.add_job([job0], 'connection_mode', idx=0)
        job1 = MDSjob('open', tree,  int(shot))
        ana.add_job([job1], 'connection')
        ana.add_job([MDSjob('value', expr)], 'value')

#       ana.start_job(pool, wid)
        ana_group = AnaGroup(True, [ana])
        gc = GlobalCounter(0,  td)
        ana_group.gc = gc
        ana_group.func = self.call_back
#       self.queue = Queue.Queue()
        pool.submit_ana_group(ana_group)

#       self.queue.get(True, 5)
#       ana_group, idx = pool.check_ana_group_done()
#       ana = ana_group[0]
#       if ana.check_finished():
        count = 0
        while count < 10:
            time.sleep(1)
            if ana.status:
                break
            count = count + 1
        if ana.status:
            return ana.result['value']

        print('Error in reading data')
        return None

    def call_back(self, *args, **kargs):
        pass
#       print 'call_back is called'
#       self.queue.put('')


class NETCDFfileVVV(NamelistVVV):
    def get_drag_text2(self, key):
        ''' 
        text for dnd from var viewer
        '''
        text = self._td.get_full_path()+self._path+'.var["'+key+'"]'
        return text

    def OnCompareItems(self, t1, t2):
        klist = t1[1][0]
        p1 = t1[0]._var0
        for key in klist[:-1]:
            p1 = p1[key]
        p2 = t2[0]._var0
        klist = t2[1][0]
        for key in klist[:-1]:
            p2 = p2[key]

        return (list(p1.keys()).index(t1[1][0][-1]) -
                list(p2.keys()).index(t2[1][0][-1]))

    def tree_viewer_menu(self):
        if self._content()._data_loaded:
            m = [("Show as plot",  self.onScope1D, None),
                 ("Show as image",  self.onScope2Di, None),
                 ("Show as contour",  self.onScope2Dc, None),
                 ("Show as surface",  self.onScope2Ds, None),
                 ("Export Content Object",  self.onExportContent, None),
                 ("Erase data on Memory",  self.onEraseDataOnMem, None),
                 ("Copy path",  self.onCopyPath, None), ]
        else:
            m = [("Show as plot",  self.onScope1D, None),
                 ("Show as image",  self.onScope2Di, None),
                 ("Show as contour",  self.onScope2Dc, None),
                 ("Show as surface",  self.onScope2Ds, None),
                 ("Export Content Object",  self.onExportContent, None),
                 ("Load data to Memory",  self.onLoadData2Mem, None),
                 ("Copy path",  self.onCopyPath, None), ]

        return m

    def onScope1D(self, evt):
        from ifigure.mto.fig_plot import FigPlot
        y = self._content().nc_eval(self._td)
        obj = FigPlot(y)
        self._on_scope(obj, False)

    def onScope2Di(self, evt):
        from ifigure.mto.fig_image import FigImage
        z = self._content().nc_eval(self._td)
        obj = FigImage(z)
        self._on_scope(obj, False)

    def onScope2Dc(self, evt):
        from ifigure.mto.fig_contour import FigContour
        z = self._content().nc_eval(self._td)
        obj = FigContour(z)
        self._on_scope(obj, False)

    def onScope2Ds(self, evt):
        from ifigure.mto.fig_surface import FigSurface
        z = self._content().nc_eval(self._td)
        obj = FigSurface(z)
        self._on_scope(obj, True)

    def _on_scope(self, obj, threed=False):
        #       print self.getvar('mds_path')
        #       print self._td.get_full_path(), self._path
        from ifigure.interactive import figure

        book = figure()
        viewer = book.get_root_parent().app.find_bookviewer(book)

        p = book.get_page(0)
        a = p.get_axes(0)
        a.set_3d(threed)
        name = a.get_next_name(obj.get_namebase())
        a.add_child(name, obj)
        obj.realize()

        ifigure.events.SendPVAddFigobj(a)
        ifigure.events.SendChangedEvent(a, w=viewer.canvas)

    def onLoadData2Mem(self, evt):
        self._content().load2mem(self._td)

    def onEraseDataOnMem(self, evt):
        self._content().erase_mem()

    def onExportContent(self, evt):
        app0 = wx.GetApp().TopWindow
        ret, m = dialog.textentry(app0,
                                  "Enter variable name", "Export Content Object", 'content')
        if not ret:
            return None
        self._td.write2shell(self._content(), m)

    def onCopyPath(self, evt):
        full_path = self._td.get_full_path()
        text = full_path + self._path+'.nc_eval('+full_path+')'

        SetText2Clipboard(text)


class NETCDFfile(Namelist):
    class_version = 1.0

    def __init__(self, *arg, **karg):
        self.var = collections.OrderedDict()
        self.nc_path = []
        self.version = self.__class__.class_version
        self._data = None
        self._data_loaded = False

        super(PyContents, self).__init__(*arg, **karg)

    def __setitem__(self, key, value):
        # slicing weakref with [:] gives the 2nd...why?
        if isinstance(value, collections.OrderedDict):
            value2 = NETCDFfile(value)
        else:
            value2 = value

        super(PyContents, self).__setitem__(key, value2)

    def show_contents(self, tree, td, item, base=None, keylist=None):
        ''' 
        proj viwer helper method
        '''
        img = td.get_classimage()
        if base is None:
            base = self
        if keylist is None:
            keylist = []
        self.make_contents_branch(tree, td, item, img)

    def make_contents_branch(self, tree, td, pitem, img,
                             keylist=None):
        if keylist is None:
            keylist = []
        for key in list(self.keys()):
            tree.SetItemHasChildren(pitem)
            pitem2 = tree.AppendItem(pitem, key, img)
            vv_val = self[key].get_varviewer_value(td, keylist+[key])
            tree.SetPyData(pitem2, (td, (keylist+[key], self[key]),
                                    vv_val))
            # self[key].nc_path=keylist+[key]
            if isinstance(self[key], dict):
                keylist2 = [x for x in keylist]
                keylist2.append(key)
                self[key].make_contents_branch(tree, td, pitem2,
                                               img, keylist=keylist2)

    def get_varviewer_value(self, td, keylist):
        val = NETCDFfileVVV(self.var, {})
        txt = '"]["'.join(keylist)
        if not isinstance(td, weakref.ProxyType):
            td = weakref.proxy(td)
        val._td = td  # weakref.proxy(td)
        val._content = weakref.ref(self)
        val._path = '[:]["'+txt+'"]'
        return val

    def get_drag_text1(self, pydata):
        ''' 
        text for dnd from proj viewer
        '''
        txt = '","'.join(pydata[1][0])
        return pydata[0].get_full_path()+'.get_contents("'+txt+'")'

    def nc_eval(self, td):
        from netCDF4 import Dataset

        fpath = td.path2fullpath(modename='nc4_pathmode',
                                 pathname='nc4_path')
        g0 = Dataset(fpath, 'r', format='NETCDF4')
        g = g0
        for i in range(len(self.nc_path)-1):
            if hasattr(g, self.nc_path[i]):
                g = getattr(g, self.nc_path[i])
            else:
                g = g.__getitem__(self.nc_path[i])
        if g is not None:
            v = g[self.nc_path[-1]][:]
            g0.close()
            return v
        else:
            g0.close()
            return None

    def load2mem(self, td):
        value = self.nc_eval(td)
        self._data = value
        self._data_loaded = True

    def erase_mem(self):
        self._data = None
        self._data_loaded = False

    def eval(self, td):
        return self.nc_eval(td)


class TSCInputFile(Namelist):
    class_version = 1.0

    def show_contents(self, tree, td, item, base=None, keylist=None):
        ''' 
        proj viwer helper method
        '''
        img = td.get_classimage()

        if base is None:
            base = self
        if keylist is None:
            keylist = []
        pitem = item
        for key in list(base.keys()):
            tree.SetItemHasChildren(pitem)
            if 'name' in self[key]:
                extra = '('+self[key]['name']+')'
            else:
                extra = ''

            pitem2 = tree.AppendItem(pitem, key+extra, img)
            vv_val = self.get_varviewer_value(td, key)
            tree.SetPyData(pitem2, (td, (key, base[key]), vv_val))



import wx
import wx.stc as stc


class Tipwindow(wx.MiniFrame):
    def __init__(self, *args, **kargs):
        wx.MiniFrame.__init__(self, *args,
                              style=wx.CAPTION |
                              wx.CLOSE_BOX |
                              wx.MINIMIZE_BOX |
                              wx.RESIZE_BORDER |
                              wx.FRAME_FLOAT_ON_PARENT)
        vbox = wx.BoxSizer(wx.VERTICAL)

        panel = wx.Panel(self, wx.ID_ANY)
        panel.SetSizer(vbox)
       # self.tip = wx.StaticText(panel, -1,
       #                          style=wx.TE_MULTILINE)#|wx.TE_READONLY|wx.VSCROLL)
       # self.tip.SetBackgroundColour((255,255,0))
        self.tip = stc.StyledTextCtrl(panel, -1)
        self.tip.SetMarginWidth(0, 0)
        self.tip.SetMarginWidth(1, 0)
        self.tip.SetMarginWidth(2, 0)
        self.tip.SetReadOnly(True)
        self.tip.StyleSetBackground(wx.stc.STC_STYLE_DEFAULT, (255, 255, 0))

        vbox.Add(self.tip, 1, wx.EXPAND | wx.ALL, 5)
        self.Bind(wx.EVT_CLOSE, self.onWindowClose)
        self.Bind(wx.EVT_ACTIVATE, self.onActivate)
        #panel.Bind(wx.EVT_SET_FOCUS, self.onSetFocus)
        #panel.Bind(wx.EVT_KILL_FOCUS, self.onKillFocus)
        #self.tip.Bind(wx.EVT_SET_FOCUS, self.onSetFocus)
        #self.tip.Bind(wx.EVT_KILL_FOCUS, self.onKillFocus)
        self.Hide()

#    def update(self, func):
    def update(self, name, argspec, tip):
        #        if hasattr(func, '__doc__'):
        try:
            #                self.tip.SetLabel(func.__doc__)
            self.tip.SetReadOnly(False)
            self.tip.SetText(tip)
            self.tip.StyleClearAll()
            self.tip.SetReadOnly(True)
        except:
            pass

    def onActivate(self, e):
        return
        if e.GetActive():
            self.SetTransparent(255)
        else:
            self.SetTransparent(120)

    def onWindowClose(self, e):
        self.Hide()
        e.Veto()



'''
simpe package/module management tool for piscopelib 

piscopelib is a libray located in .ifigure_rc/.local/site-packages/piscopelib


'''
import wx
from ifigure.utils.edit_list import DialogEditList


def setup(package='', root='', path='setup_scripts',
          del_scripts=True, model=None):

    from ifigure.mto.hg_support import load_subtree_hg, dlg_s
    from ifigure.events import SendSelectionEvent

    app = wx.GetApp().TopWindow
    proj = app.proj
    name = 'setup_scritps'

    if root == '' or path == '':
        if path == '':
            path = 'setup_scripts'
        list6 = [
            ["root repository", None, 404, dlg_s()],
            ["source ", path, 0], ]
        value = DialogEditList(list6, modal=True,
                               style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER,
                               tip=None,
                               parent=app,
                               title='Select Setup Scripts Repo.')
        if not value[0]:
            return
        root = str(value[1][0])
        path = str(value[1][1])

    if not proj.has_owndir():
        proj.mk_owndir()

    if proj.has_child(name):
        name = proj.get_next_name(name)
    obj = load_subtree_hg(proj, root=root, path=path, name=name, overwrite=False,
                          run_setup=False, launch_gui=False)  # , src = 'setup_scripts')

    separator = '---------'
    if package == '':
        names = obj.get_childnames()
        names = sorted(names)
        names.extend([separator, 'setup scripts'])
        setting = {"style": wx.CB_READONLY,
                   "choices": names}
        list6 = [[None, "Select scripts to run ", 2, None],
                 ["setup scripts", names[0], 4, setting], ]
#             [None, del_scripts, 3, {"text":'Delete scripts folder after the run'}],]

        value = DialogEditList(list6, modal=True,
                               style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER,
                               tip=None,
                               parent=app,
                               title='Select Setup Scripts')
        if not value[0]:
            if del_scripts:
                obj.destroy()
            else:
                app.proj_tree_viewer.update_widget()
                SendSelectionEvent(obj)
            return
        package = str(value[1][1])
#        del_scripts = value[1][2]

    if (package != 'setup scripts') and (package != separator):
        script = obj.get_child(name=package)
        if model is None:
            model = proj.onAddModel()
        new_model = script(model)

        if del_scripts:
            obj.destroy()

        app.proj_tree_viewer.update_widget()
        if new_model is not None:
            SendSelectionEvent(new_model)
        return model
    else:
        app.proj_tree_viewer.update_widget()
        SendSelectionEvent(obj)
        return obj



from __future__ import print_function
#
#  Name   : py_fiile
#
#          mto to hold a file
#
#          py_file : generic (non-text) files
#          py_text : text files (add edit menu)
#
#          there are five file path modes
#
#              std:  in ifigure (root = ifigure)
#              wdir: in project dir (root = wdir)
#              owndir: in object dir (root = self.owndir())
#              home: in home dir (root = home dirctory)
#              abs : elsewhere (root = '/')
#
#          the order of check priority is as follows
#              owndir
#              wdir
#              std
#              home
#              abs
#
#          among these fivce, std|home|abs is external file
#          mode.
#
#          it also check time stamp of file for version
#          control
#
#
#
#
#  Author :
#         Syun'ichi Shiraiwa
#  E-mail :
#         shiraiwa@psfc.mit.edu
#
#
#  History : 0.5 sometime around Apr. 2012
#            0.6 stop() command
#            0.7 8/30/12 keyborad interupt
#
from ifigure.mto.treedict import TreeDict
import ifigure.utils.cbook as cbook
import ifigure
import ifigure.events
import ifigure.widgets.dialog as dialog
from ifigure.mto.fileholder import FileHolder
import os
import logging
import wx
import threading
import types
import shutil
import multiprocessing


class PyFile(TreeDict, FileHolder):
    def __init__(self, parent=None, script='', src=None):
        super(PyFile, self).__init__(parent=parent, src=src)
        self._can_have_child = False
        self._has_private_owndir = False
        self.setvar("file_pathmode", '')
        self.setvar("file_path", '')
        self.setvar("file_ext", '')

    @classmethod
    def get_namebase(self):
        return 'file'

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'file.png')
        return [idx1]

    @classmethod
    def can_have_child(self, child=None):
        return False

    def tree_viewer_menu(self):
     # return MenuString, Handler, MenuImage
        app = self.get_root_parent().app
        m = [('+File', None, None), ]
        if self.getvar('file_ext') in app.helper.setting['ext'].split(','):
            m.append(('Open', self.onOpenWithHelper, None))
        m = m + [
            ('Set File...', self.onSetFile, None),
            ('Import File...', self.onImportFile, None),
            ('Export File...', self.onExportFile, None),
            ('!', None, None),
            ('---', None, None)] + \
            super(PyFile, self).tree_viewer_menu()
        return m

#    def onRename(self, evt):
#        super(PyScript, self).onRename(evt)
    def onOpenWithHelper(self, e):
        fpath = self.path2fullpath('file_pathmode', 'file_path')
        if fpath == '':
            return
        app = self.get_root_parent().app
        ext = self.getvar('file_ext')

        com = app.helper.setting[ext]
        root = os.path.dirname(os.path.dirname(ifigure.__path__[0]))
        bin = os.path.join(root, 'bin')
        com = com.replace('{bin}', bin)
#        com = com.replace('{0}',   '"'+fpath+'"')
        com = com.replace('{0}',   fpath)
        import subprocess
        import shlex
        print(com)
        subprocess.Popen(com, shell=True)
#        subprocess.Popen(shlex.split(com))
#        import os
#        os.system(

    def onEdit(self, e):
        fpath = self.path2fullpath('file_pathmode', 'file_path')
        if fpath == '':
            return
        ifigure.events.SendEditFileEvent(self,
                                         w=e.GetEventObject(),
                                         file=fpath)

    def onSetFile(self, e):
        file = dialog.read(None, message="Select file")
        self.setfile(file)

    def setfile(self, file):
        if (file != '' and
                os.path.exists(file)):
            self.set_path_pathmode(file)
            self.store_mtime('file_mtime')

    def onImportFile(self, e):
        '''
        import file from outside project directory
        pathmode will be set to 'owndir'
        '''

        file = dialog.read(None, message="Select file")
        if file == '':
            return
        mode, path = self.fullpath2path(file)
        if (mode == 'wdir' or mode == 'owndir'):
            m = 'Import should import from somewhere outside project directory',
            ret = dialog.message(None, message=m,
                                 title='Import error')
            return
        self.import_file(file)

    def onExportFile(self, e):
        opath = self.path2fullpath('file_pathmode', 'file_path')

        file = dialog.write(None, defaultfile=opath,
                            message="Enter file name")
        if file == '':
            return
        mode, path = self.fullpath2path(file)
        if (mode == 'wdir' or mode == 'owndir'):
            m = 'Export should export to somewhere outside project directory',
            ret = dialog.message(None, message=m,
                                 title='Import error')
            return

        shutil.copyfile(opath, file)

    def export_file(self, file):
        '''
        export scirpt file to file.
        the destination should be outside
        of wdir
        '''
        path = self.path2fullpath('file_pathmode', 'file_path')
        mode, path0 = self.fullpath2path(file)
        if mode == 'wdir':
            # do not write inside proj directory
            print('can not export to wdir')
            return
        if mode == 'owndir':
            # do not write inside proj directory
            print('can not export to owndir')
            return

        shutil.copyfile(path, file)

    def import_file(self, file):
        '''
        import *.py file from outside of wdir
        the location of original file will be
        saved to "orgpath", "orgpathmode"
        '''
        mode, path = self.fullpath2path(file)
        base, ext = self.split_ext(path)

        if mode == 'wdir':
            # do nothing in this case
            print('can not import from wdir')
            return
        if mode == 'owndir':
            # do not write inside proj directory
            print('can not import from owndir')
            return

        self.setvar("file_orgpath", str(path))
        self.setvar("file_orgpathmode", str(mode))
        self.setvar("file_ext", str(ext))

        if not self.has_owndir():
            self.mk_owndir()
        file2 = os.path.join(self.owndir(), self.name+'.'+ext)
        shutil.copyfile(file, file2)

        self.setvar("file_pathmode", 'owndir')
        self.setvar("file_path", self.name+'.'+ext)
        self.store_mtime('file_mtime')

    def rename(self, new, ignore_name_readonly=False):
        path = self.getvar("file_path")
        base, ext = self.split_ext(path)
        ooname = self.path2fullpath('file_pathmode', 'file_path')
        super(PyFile, self).rename(
            new, ignore_name_readonly=ignore_name_readonly)

        oname = self.path2fullpath('file_pathmode', 'file_path')
        self.setvar("file_path", self.name+'.'+ext)
        nname = self.path2fullpath('file_pathmode', 'file_path')

        if oname != nname:
            os.rename(oname, nname)
            self.set_path_pathmode(nname)
            param = {"oldname": ooname, "newname": nname}
            op = 'rename'
            ifigure.events.SendFileChangedEvent(self, operation=op,
                                                param=param)

#           ifigure.events.SendChangedEvent(self)

    # utility to convert fullpath to path
    def set_path_pathmode(self, path,
                          modename='file_pathmode',
                          pathname='file_path',
                          extname='file_ext'):
        ''' 
        this is to overwrite default var
        '''
        return super(PyFile, self).set_path_pathmode(path, modename,
                                                     pathname, extname)

    def store_mtime(self, name='file_mtime', modename='file_pathmode', pathname='file_path'):
        ''' 
        this is to overwrite default var
        '''
        return super(PyFile, self).store_mtime('file_mtime',
                                               'file_pathmode',
                                               'file_path')

    def path2fullpath(self, modename='pathmode', pathname='path'):
        ''' 
        this is to overwrite default var
        '''
        return super(PyFile, self).path2fullpath('file_pathmode', 'file_path')

    def init_after_load(self, olist, nlist):
        try:
            if self.isExternalFileNewer('file_mtime', 'file_pathname', 'file_path'):
                self._status = '!! external file changed'
            else:
                self._status = ''
        except:
            self._status = '!! filenew check error'


class PyText(PyFile):
    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'text.png')
        return [idx1]

    @classmethod
    def get_namebase(self):
        return 'text'

    def pv_kshortcut(self, key):
        '''
        define keyborad short cut on project viewer
        must return a method responding key
        '''
        if key == wx.WXK_RETURN:
            return self.onEdit
        return None

    def tree_viewer_menu(self):
     # return MenuString, Handler, MenuImage
        return [('+File', None, None),
                ('Edit', self.onEdit, None),
                ('Set File...', self.onSetFile, None),
                ('Import File...', self.onImportFile, None),
                ('Export File...', self.onExportFile, None),
                ('Export Text to Shell',  self.onExportToShell, None),
                ('!', None, None),
                ('---', None, None)] + \
            super(PyFile, self).tree_viewer_menu()

    def onExportToShell(self, evt):
        fpath = self.path2fullpath('file_pathmode', 'file_path')
        fid = open(fpath, 'r')
        lines = fid.readlines()
        fid.close()

        id = evt.GetEventObject()
        app = id.GetTopLevelParent()
        ret, new_name = dialog.textentry(app,
                                         "Enter the name of new variable",
                                         "Export Text File to Shell", "")
        if ret:
            flag = self.write2shell(lines, new_name)
            app.shell.SendShellEnterEvent()
        e.Skip()  # call update project tree widget
